@@ -28,6 +28,8 @@ module Data.Yaml
     , Object
     , Array
     , ParseException(..)
+    , YamlException (..)
+    , YamlMark (..)
       -- * Constructors and accessors
     , object
     , array
@@ -49,6 +51,8 @@ module Data.Yaml
     , decodeFile
       -- ** Better error information
     , decodeEither
+    , decodeEither'
+    , decodeFileEither
       -- ** More control over decoding
     , decodeHelper
     ) where
@@ -65,7 +69,7 @@ import Data.ByteString (ByteString)
 import qualified Data.ByteString.Char8 as S8
 import qualified Data.Map as Map
 import System.IO.Unsafe (unsafePerformIO)
-import Control.Exception (try, throwIO, fromException, Exception)
+import Control.Exception (try, throwIO, fromException, Exception, SomeException, AsyncException)
 import Control.Monad.Trans.State
 import qualified Data.Conduit as C
 import qualified Data.Conduit.List as CL
@@ -82,6 +86,7 @@ import Data.Text.Encoding.Error (lenientDecode)
 import qualified Data.HashMap.Strict as M
 import Data.Typeable
 import Data.Attoparsec.Number
+import qualified Data.HashSet as HashSet
 
 encode :: ToJSON a => a -> ByteString
 encode obj = unsafePerformIO $
@@ -119,7 +124,14 @@ objToEvents' (Object pairs) rest =
 -- https://github.com/snoyberg/yaml/issues/24
 objToEvents' (String "") rest = EventScalar "" NoTag SingleQuoted Nothing : rest
 
-objToEvents' (String s) rest = EventScalar (encodeUtf8 s) StrTag PlainNoTag Nothing : rest
+objToEvents' (String s) rest =
+    event : rest
+  where
+    event
+        -- Make sure that special strings are encoded as strings properly.
+        -- See: https://github.com/snoyberg/yaml/issues/31
+        | s `HashSet.member` specialStrings = EventScalar (encodeUtf8 s) NoTag SingleQuoted Nothing
+        | otherwise = EventScalar (encodeUtf8 s) StrTag PlainNoTag Nothing
 objToEvents' Null rest = EventScalar "null" NullTag PlainNoTag Nothing : rest
 objToEvents' (Bool True) rest = EventScalar "true" BoolTag PlainNoTag Nothing : rest
 objToEvents' (Bool False) rest = EventScalar "false" BoolTag PlainNoTag Nothing : rest
@@ -130,6 +142,11 @@ pairToEvents (k, v) rest =
     EventScalar (encodeUtf8 k) StrTag PlainNoTag Nothing
   : objToEvents' v rest
 
+-- | Strings which must be escaped so as not to be treated as non-string scalars.
+specialStrings :: HashSet.HashSet Text
+specialStrings = HashSet.fromList $ T.words
+    "y Y yes Yes YES n N no No NO true True TRUE false False FALSE on On ON off Off OFF null Null NULL ~"
+
 -- Parsing
 
 data ParseException = NonScalarKey
@@ -138,6 +155,8 @@ data ParseException = NonScalarKey
                                       , _expected :: Maybe Event
                                       }
                     | InvalidYaml (Maybe YamlException)
+                    | AesonException String
+                    | OtherParseException SomeException
     deriving (Show, Typeable)
 instance Exception ParseException
 
@@ -254,11 +273,11 @@ parseM a front = do
             let al  = M.insert s o front
                 al' = if s == pack "<<"
                          then case o of
-                                  Object l  -> M.union al l
-                                  Array l -> M.union al $ foldl merge' M.empty $ V.toList l
+                                  Object l  -> M.union front l
+                                  Array l -> M.union front $ foldl merge' M.empty $ V.toList l
                                   _          -> al
                          else al
-            parseM a $ M.insert s o al'
+            parseM a al'
     where merge' al (Object om) = M.union al om
           merge' al _           = al
 
@@ -274,11 +293,46 @@ decodeFile :: FromJSON a
            -> IO (Maybe a)
 decodeFile fp = decodeHelper (Y.decodeFile fp) >>= either throwIO (return . either (const Nothing) id)
 
+-- | A version of 'decodeFile' which should not throw runtime exceptions.
+--
+-- Since 0.8.4
+decodeFileEither
+    :: FromJSON a
+    => FilePath
+    -> IO (Either ParseException a)
+decodeFileEither fp = do
+    x <- try $ C.runResourceT $ flip evalStateT Map.empty $ Y.decodeFile fp C.$$ parse
+    case x of
+        Left e
+            | Just pe <- fromException e -> return $ Left pe
+            | Just ye <- fromException e -> return $ Left $ InvalidYaml $ Just (ye :: YamlException)
+            | shouldBeCaught e -> return $ Left $ OtherParseException e
+            | otherwise -> throwIO e
+        Right y ->
+            return $ case parseEither parseJSON y of
+                Left s -> Left $ AesonException s
+                Right v -> Right v
+  where
+    -- FIXME this function needs more thought
+    shouldBeCaught e
+        | Just (_ :: AsyncException) <- fromException e = False
+        -- Would be nice... | Just (_ :: Timeout) <- fromException e = False
+        | otherwise = True
+
 decodeEither :: FromJSON a => ByteString -> Either String a
 decodeEither bs = unsafePerformIO
                 $ fmap (either (Left . show) id)
                 $ decodeHelper (Y.decode bs)
 
+-- | More helpful version of 'decodeEither' which returns the 'YamlException'.
+--
+-- Since 0.8.3
+decodeEither' :: FromJSON a => ByteString -> Either ParseException a
+decodeEither' = either Left (either (Left . AesonException) Right)
+              . unsafePerformIO
+              . decodeHelper
+              . Y.decode
+
 decodeHelper :: FromJSON a
              => C.Source Parse Y.Event
              -> IO (Either ParseException (Either String a))
