@@ -2,6 +2,7 @@
 {-# LANGUAGE PackageImports #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE TemplateHaskell #-}
 
 import qualified Text.Libyaml as Y
 import qualified Data.ByteString.Char8 as B8
@@ -21,6 +22,21 @@ import Data.Yaml (object, array, (.=))
 import Data.Maybe
 import qualified Data.HashMap.Strict as M
 import qualified Data.Text as T
+import Data.Aeson.TH
+import Data.Text (Text)
+import Data.Vector (Vector)
+import qualified Data.Vector as V
+import Data.HashMap.Strict (HashMap)
+import qualified Data.HashMap.Strict as HM
+
+data TestJSON = TestJSON
+              { string :: Text
+              , number :: Int
+              , anArray  :: Vector Text
+              , hash   :: HashMap Text Text
+              } deriving (Show, Eq)
+
+deriveJSON defaultOptions ''TestJSON
 
 main :: IO ()
 main = hspec $ do
@@ -62,29 +78,52 @@ main = hspec $ do
         it "parses when titlecase" caseTitlecaseBool
     describe "empty input" $ do
         it "doesn't crash" caseEmptyInput
+    describe "alias stripping" $ do
+        it "works" caseStripAlias
     describe "nulls" $ do
         checkNull "null"
         checkNull "Null"
         checkNull "NULL"
         checkNull "~"
         checkNull ""
+
     describe "pretty output" $ do
         it "simple nulls" $ D.encode (object ["foo" .= D.Null]) `shouldBe` "foo: null\n"
         it "simple numbers" $ D.encode (object ["foo" .= (4 :: Int)]) `shouldBe` "foo: 4\n"
         it "True" $ D.encode (object ["foo" .= True]) `shouldBe` "foo: true\n"
         it "False" $ D.encode (object ["foo" .= False]) `shouldBe` "foo: false\n"
         it "simple string" $ D.encode (object ["foo" .= ("bar" :: T.Text)]) `shouldBe` "foo: bar\n"
+
     describe "special keys" $ do
         let tester key = it (T.unpack key) $
                 let value = object [key .= True]
                  in D.decode (D.encode value) `shouldBe` Just value
         mapM_ tester specialStrings
+
     describe "special values" $ do
         let tester value = it (T.unpack value) $
                 let value' = object ["foo" .= value]
                  in D.decode (D.encode value') `shouldBe` Just value'
         mapM_ tester specialStrings
 
+    describe "yamlFileJSONparse" $
+        it "loads YAML through JSON into Haskell data" $ do
+          tj <- either (error . show) id `fmap` D.decodeFileEither "test/json.yaml"
+          tj `shouldBe` TestJSON
+                          { string = "str"
+                          , number = 2
+                          , anArray = V.fromList ["a", "b"]
+                          , hash = HM.fromList [("key1", "value1"), ("key2", "value2")]
+                          }
+
+    describe "round-tripping of special scalars" $ do
+        let special = words "y Y On ON false"
+        forM_ special $ \w -> it w $
+            let v = object ["word" .= w]
+             in D.decode (D.encode v) `shouldBe` Just v
+        it "no tags" $ D.encode (object ["word" .= ("true" :: String)]) `shouldBe` "word: 'true'\n"
+
+
 specialStrings :: [T.Text]
 specialStrings =
     [ "fo\"o"
@@ -365,3 +404,19 @@ checkNull x =
     it ("null recognized: " ++ show x) assertion
   where
     assertion = Just (object [("foo", D.Null)]) @=? D.decode (B8.pack $ "foo: " ++ T.unpack x)
+
+caseStripAlias :: Assertion
+caseStripAlias =
+    D.decode src @?= Just (object
+        [ "Default" .= object
+            [ "foo" .= (1 :: Int)
+            , "bar" .= (2 :: Int)
+            ]
+        , "Obj" .= object
+            [ "foo" .= (1 :: Int)
+            , "bar" .= (2 :: Int)
+            , "key" .= (3 :: Int)
+            ]
+        ])
+  where
+    src = "Default: &def\n  foo: 1\n  bar: 2\nObj:\n  <<: *def\n  key: 3\n"
